#include "followop.h"

UInt16 follow_straight (UInt16 where)
{
  UChar op = Op6502 (where);

  return where + addr_info[optable[op].addr_mode].size;
}

UInt16 follow_BBRi (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar test_char = Op6502 (Op6502 (where + 1));
  UChar nb_bit = op >> 4;
  UChar size = addr_info[optable[op].addr_mode].size;

  test_char >>= nb_bit;

  if (!(test_char & 0x1))	// no jump
    return where + size;

  // jump
  return where + addr_info[optable[op].addr_mode].size
    + (SChar) Op6502 (where + size - 1);

}

UInt16 follow_BCC (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  if (M.P & C_FLAG)		// no jump
    return where + size;

  // jump
  return where + size + (SChar) Op6502 (where + size - 1);
}

UInt16 follow_BBSi (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar test_char = Op6502 (Op6502 (where + 1));
  UChar nb_bit = op >> 4;
  UChar size = addr_info[optable[op].addr_mode].size;

  test_char >>= nb_bit;

  if ((test_char & 0x1))	// no jump
    return where + size;

  // jump
  return where + addr_info[optable[op].addr_mode].size
    + (SChar) Op6502 (where + size - 1);

}

UInt16 follow_BCS (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  if (!(M.P & C_FLAG))		// no jump
    return where + size;

  // jump
  return where + size + (SChar) Op6502 (where + size - 1);
}

UInt16 follow_BEQ (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  if (M.ZF)			// no jump
    return where + size;

  // jump
  return where + size + (SChar) Op6502 (where + size - 1);
}

UInt16 follow_BNE (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  if (!(M.ZF))			// no jump
    return where + size;

  // jump
  return where + size + (SChar) Op6502 (where + size - 1);
}

UInt16 follow_BMI (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  if (M.NF & N_FLAG)		// no jump
    return where + size;

  // jump
  return where + size + (SChar) Op6502 (where + size - 1);
}

/* TEST : remove the log stuff */
UInt16 follow_BPL (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  Log ("BPL hit at 0X%04X\nsize of op = %d\n", where, size);

  if (!(M.NF & N_FLAG))		// no jump
    {
      Log ("Won't jump to go to 0X%04X\n", where + size);
      return where + size;
    }

  Log ("Will jump to 0X%04X\n",
       where + size + (SChar) Op6502 (where + size - 1));

  // jump
  return where + size + (SChar) Op6502 (where + size - 1);
}

UInt16 follow_BRA (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  return where + size		// always jump
    + (SChar) Op6502 (where + size - 1);
}

UInt16 follow_BSR (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  if (running_mode == STEPPING)	// no jump
    return where + size;

  // jump
  return where + size + (SChar) Op6502 (where + size - 1);
}

UInt16 follow_BVC (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  if (M.VF & V_FLAG)		// no jump
    return where + size;

  // jump
  return where + size + (SChar) Op6502 (where + size - 1);
}

UInt16 follow_BVS (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  if (!(M.VF & V_FLAG))		// no jump
    return where + size;

  // jump
  return where + size + (SChar) Op6502 (where + size - 1);
}

UInt16 follow_JMPabs (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  return Op6502 (where + size - 2) + 256 * Op6502 (where + size - 1);

}

UInt16 follow_JMPindir (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  UInt16 indir = Op6502 (where + size - 2) + 256 * Op6502 (where + size - 1);

  return Op6502 (indir) + 256 * Op6502 (indir + 1);

}

UInt16 follow_JMPindirX (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  UInt16 indir = Op6502 (where + size - 2) + 256 * Op6502 (where + size - 1);

  indir += M.X;

  return Op6502 (indir) + 256 * Op6502 (indir + 1);

}

UInt16 follow_JSR (UInt16 where)
{
  UChar op = Op6502 (where);
  UChar size = addr_info[optable[op].addr_mode].size;

  if (running_mode == STEPPING)
    return where + size;	// no jump

  return Op6502 (where + size - 2) + 256 * Op6502 (where + size - 1);

}

UInt16 follow_RTI (UInt16 where)
{
  return RAM[SP + M.S + 2] + 256 * RAM[SP + M.S + 3];
}

UInt16 follow_RTS (UInt16 where)
{
  return RAM[SP + M.S + 1] + 256 * RAM[SP + M.S + 2] + 1;
}
